#!/usr/bin/env ruby
# freeze_string_literals: true

require 'optparse'
require 'yaml'

OPTIONS_FILE = File.expand_path('config.yaml', __dir__)

unless File.exist?(OPTIONS_FILE)
  puts "Configuration file '#{OPTIONS_FILE}' does not exist."
  exit 1
end

options = YAML.load_file(OPTIONS_FILE)

unless options.is_a?(Hash)
  puts 'The configuration file is in wrong format, please use the correct format'
  exit 1
end

unless options.key?('pid_file')
  puts "Configuration file does not contain PID file location in 'pid_file'"
  exit 1
end

begin
  pid_file = File.open(options['pid_file'], 'w')
rescue
  puts "Unable to open PID file '#{options['pid_file']}'"
  puts 'Please specify another location for the file'
  exit 1
end

unless options.key?('output_directory')
  puts "Configuration file does not contain the log location in 'output_directory'"
  exit 1
end

unless Dir.exist?(options['output_directory'])
  puts "The output directory you have specified does not exist: '#{options['output_directory']}'"
  exit 1
end

begin
  test_file_name = File.join(options['output_directory'], 'test-file')
  test_file = File.open(test_file_name, 'w')
  test_file.close
  File.delete(test_file_name)
rescue
  puts "Unable to write data to the directory '#{options['output_directory']}'"
  exit 1
end

unless options.key?('sleep_duration')
  puts "You must provide the sleep duration in the 'sleep_duration' property"
  exit
end

sleep_duration = options['sleep_duration'].to_i

if sleep_duration <= 0
  puts 'Sleep duration must be positive integer number'
  exit 1
end

child_id = fork
if child_id
  puts "Started service in the background with id: #{child_id}"
  pid_file.puts(child_id.to_s)
  pid_file.close
  Process.detach(child_id)
  exit 0
end

continue = true
Signal.trap('TERM') do
  puts 'Shutting down'
  continue = false
end

log_file_index = 1
while continue
  file_name = format('log-file-%05d', log_file_index)
  File.open(File.join(options['output_directory'], file_name), 'w') do |log_file|
    10.times do
      break unless continue

      process_count = `ps aux | wc -l`
      memory_info = `free -h`
      log_file.puts("System time: #{Time.now.strftime('%F-%H-%M-%S')}")
      log_file.puts("Process count: #{process_count}")
      log_file.puts('Memory information')
      log_file.puts(memory_info)
      sleep(sleep_duration)
    end
  end
  log_file_index += 1
end
